@@ -15,7 +15,6 @@
 #include <string>
 
 namespace pl::ptrn {
-
     using namespace ::std::literals::string_literals;
 
     class IInlinable {
@@ -34,7 +33,7 @@ namespace pl::ptrn {
         virtual void setEntries(const std::vector<std::shared_ptr<Pattern>> &entries) = 0;
 
         [[nodiscard]] virtual std::shared_ptr<Pattern> getEntry(size_t index) const = 0;
-        virtual void forEachEntry(u64 start, u64 end, const std::function<void(u64, Pattern*)> &callback) = 0;
+        virtual void forEachEntry(u64 start, u64 end, const std::function<void(u64, const std::shared_ptr<Pattern>&)> &callback) = 0;
 
         [[nodiscard]] virtual size_t getEntryCount() const = 0;
 
@@ -62,7 +61,7 @@ namespace pl::ptrn {
         friend class core::Evaluator;
     };
 
-    class Pattern {
+    class Pattern : public std::enable_shared_from_this<Pattern> {
     public:
         constexpr static u64 MainSectionId          = 0x0000'0000'0000'0000;
         constexpr static u64 HeapSectionId          = 0xFFFF'FFFF'FFFF'FFFF;
@@ -83,7 +82,7 @@ namespace pl::ptrn {
 
         }
 
-        Pattern(const Pattern &other) {
+        Pattern(const Pattern &other) : std::enable_shared_from_this<Pattern>(other) {
             this->m_evaluator = other.m_evaluator;
             this->m_offset = other.m_offset;
             this->m_endian = other.m_endian;
@@ -116,7 +115,14 @@ namespace pl::ptrn {
             }
         }
 
-        virtual std::unique_ptr<Pattern> clone() const = 0;
+        virtual std::shared_ptr<Pattern> clone() const = 0;
+        std::shared_ptr<Pattern> reference() {
+            auto weakPtr = weak_from_this();
+            if (weakPtr.expired()) {
+                core::err::E0001.throwError("Cannot call shared_from_this if this is not shared.");
+            }
+            return shared_from_this();
+        }
 
         [[nodiscard]] u64 getOffset() const { return this->m_offset; }
         [[nodiscard]] virtual u128 getOffsetForSorting() const { return this->getOffset() << 3; }
@@ -203,7 +209,8 @@ namespace pl::ptrn {
             else return this->m_endian.value_or(this->m_evaluator->getDefaultEndian());
         }
         virtual void setEndian(std::endian endian) {
-            if (this->isLocal()) return;
+            if (m_section == HeapSectionId || m_section == PatternLocalSectionId || m_section == InstantiationSectionId)
+                return;
 
             this->m_endian = endian;
         }
@@ -255,24 +262,35 @@ namespace pl::ptrn {
 
                 auto result = this->formatDisplayValue();
                 this->m_cachedDisplayValue = std::make_unique<std::string>(result);
+                this->m_validDisplayValue = true;
 
                 return result;
             } catch(std::exception &e) {
                 this->m_cachedDisplayValue = std::make_unique<std::string>(e.what());
+                this->m_validDisplayValue = false;
+
                 return *this->m_cachedDisplayValue;
             }
         }
 
-        [[nodiscard]] virtual std::string toString() const {
+        [[nodiscard]] bool hasValidFormattedValue() const {
+            return this->m_validDisplayValue;
+        }
+
+        [[nodiscard]] virtual std::string toString() {
             auto result = fmt::format("{} {} @ 0x{:X}", this->getTypeName(), this->getVariableName(), this->getOffset());
 
-            return this->callUserFormatFunc(this->getValue(), true).value_or(result);
+            try {
+                return this->callUserFormatFunc(this->getValue(), true).value_or(result);
+            } catch (std::exception &e) {
+                return e.what();
+            }
         }
 
         [[nodiscard]] virtual core::Token::Literal getValue() const {
-            auto clone = this->clone();
+            auto pattern = this->clone();
 
-            return this->transformValue(std::move(clone));
+            return this->transformValue(std::move(pattern));
         }
 
         [[nodiscard]] virtual std::vector<std::pair<u64, Pattern*>> getChildren() {
@@ -461,7 +479,7 @@ namespace pl::ptrn {
             this->m_cachedBytes.reset();
 
             if (auto *iterable = dynamic_cast<IIterable*>(this); iterable != nullptr) [[unlikely]] {
-                iterable->forEachEntry(0, iterable->getEntryCount(), [](u64, Pattern *pattern) {
+                iterable->forEachEntry(0, iterable->getEntryCount(), [](u64, const auto &pattern) {
                     pattern->clearByteCache();
                 });
             }
@@ -526,14 +544,14 @@ namespace pl::ptrn {
         }
 
         [[nodiscard]] const Pattern* getParent() const {
-            return m_parent;
+            return m_parent.lock().get();
         }
 
         [[nodiscard]] Pattern* getParent() {
-            return m_parent;
+            return m_parent.lock().get();
         }
 
-        void setParent(Pattern *parent) {
+        void setParent(std::shared_ptr<Pattern> parent) {
             m_parent = parent;
         }
 
@@ -543,6 +561,8 @@ namespace pl::ptrn {
             m_arrayIndex = index;
         }
 
+        [[nodiscard]] virtual std::string formatDisplayValue() = 0;
+
     protected:
         std::optional<std::endian> m_endian;
 
@@ -553,6 +573,17 @@ namespace pl::ptrn {
                 auto startHeap = this->m_evaluator->getHeap();
                 ON_SCOPE_EXIT { this->m_evaluator->getHeap() = startHeap; };
 
+                // Preserve pattern variable name
+                std::string patternName;
+                if (value.isPattern()) {
+                    patternName = value.toPattern()->getVariableName();
+                }
+                ON_SCOPE_EXIT {
+                    if (!patternName.empty())
+                        value.toPattern()->setVariableName(patternName);
+                };
+
+
                 if (auto result = transformFunc->func(evaluator, { value }); result.has_value())
                     return *result;
             }
@@ -560,8 +591,6 @@ namespace pl::ptrn {
             return value;
         }
 
-        [[nodiscard]] virtual std::string formatDisplayValue() = 0;
-        
         /**
          * @brief Calls an user-defined PL function to format the given literal (pattern), if existing. Else, returns defaultValue
         */
@@ -570,26 +599,31 @@ namespace pl::ptrn {
             if (formatterFunctionName.empty())
                 return {};
             else {
-                try {
-                    const auto function = this->m_evaluator->findFunction(formatterFunctionName);
-                    if (function.has_value()) {
-                        auto startHeap = this->m_evaluator->getHeap();
-                        ON_SCOPE_EXIT { this->m_evaluator->getHeap() = startHeap; };
+                const auto function = this->m_evaluator->findFunction(formatterFunctionName);
+                if (function.has_value()) {
+                    auto startHeap = this->m_evaluator->getHeap();
+                    ON_SCOPE_EXIT { this->m_evaluator->getHeap() = startHeap; };
 
-                        auto result = function->func(this->m_evaluator, { literal });
-                        if (result.has_value()) {
-                            if (fromCast && result->isPattern() && result->toPattern()->getTypeName() == this->getTypeName()) {
-                                return {};
-                            } else {
-                                return result->toString(true);
-                            }
+                    // Preserve pattern variable name
+                    std::string patternName;
+                    if (literal.isPattern()) {
+                        patternName = literal.toPattern()->getVariableName();
+                    }
+                    ON_SCOPE_EXIT {
+                        if (!patternName.empty())
+                            literal.toPattern()->setVariableName(patternName);
+                    };
+
+                    auto result = function->func(this->m_evaluator, { literal });
+                    if (result.has_value()) {
+                        if (fromCast && result->isPattern() && result->toPattern()->getTypeName() == this->getTypeName()) {
+                            return {};
+                        } else {
+                            return result->toString(true);
                         }
                     }
-                    return {};
-
-                } catch (core::err::EvaluatorError::Exception &error) {
-                    return error.what();
                 }
+                return {};
             }
         }
 
@@ -606,7 +640,8 @@ namespace pl::ptrn {
         }
 
     protected:
-        mutable std::unique_ptr<std::string> m_cachedDisplayValue;
+        std::unique_ptr<std::string> m_cachedDisplayValue;
+        bool m_validDisplayValue = false;
         std::unique_ptr<std::vector<u8>> m_cachedBytes;
 
     private:
@@ -615,7 +650,7 @@ namespace pl::ptrn {
         core::Evaluator *m_evaluator;
 
         std::unique_ptr<std::map<std::string, std::vector<core::Token::Literal>>> m_attributes;
-        Pattern *m_parent = nullptr;
+        std::weak_ptr<Pattern> m_parent;
         u32 m_line = 0;
 
         std::set<std::string>::const_iterator m_variableName;
