@@ -10,6 +10,7 @@
 #include <pl/patterns/pattern_bitfield.hpp>
 
 #include <bit>
+#include <wolv/utils/charconv.hpp>
 
 namespace pl::core::ast {
 
@@ -158,6 +159,9 @@ namespace pl::core::ast {
     }
 
     void applyTypeAttributes(Evaluator *evaluator, const ASTNode *node, const std::shared_ptr<ptrn::Pattern> &pattern) {
+        if (std::uncaught_exceptions() > 0)
+            return;
+
         auto attributable = dynamic_cast<const Attributable *>(node);
         if (attributable == nullptr)
             err::E0008.throwError("Attributes cannot be applied to this statement.", {}, node->getLocation());
@@ -335,7 +339,7 @@ namespace pl::core::ast {
             auto actualSize = pattern->getSize();
             if (actualSize < requestedSize) {
                 pattern->setSize(requestedSize);
-                evaluator->setReadOffset(u64(evaluator->getReadOffset() + (requestedSize - actualSize)));
+                evaluator->setReadOffset(pattern->getOffset() + requestedSize);
             }
             else if (actualSize > requestedSize)
                 err::E0004.throwError("Type size larger than expected", fmt::format("Pattern of type {} is larger than expected. Expected size {}, got {}", pattern->getTypeName(), requestedSize, actualSize), node->getLocation());
@@ -345,7 +349,7 @@ namespace pl::core::ast {
         if (!pattern->hasOverriddenColor()) {
             if (const auto &arguments = attributable->getAttributeArguments("color"); arguments.size() == 1) {
                 auto colorString = getAttributeValueAsString(arguments.front(), evaluator);
-                u32 color = strtoul(colorString.c_str(), nullptr, 16);
+                u32 color = wolv::util::from_chars<u32>(colorString, 16).value_or(0);
                 pattern->setColor(hlp::changeEndianess(color, std::endian::big) >> 8);
             } else if (auto singleColor = attributable->hasAttribute("single_color", false); singleColor) {
                 pattern->setColor(pattern->getColor());
@@ -369,6 +373,8 @@ namespace pl::core::ast {
     }
 
     void applyVariableAttributes(Evaluator *evaluator, const ASTNode *node, const std::shared_ptr<ptrn::Pattern> &pattern) {
+        if (std::uncaught_exceptions() > 0)
+            return;
 
         auto attributable = dynamic_cast<const Attributable *>(node);
         if (attributable == nullptr)
@@ -384,6 +390,8 @@ namespace pl::core::ast {
                         .bitOffset = bitfieldPattern->getBitOffset()
                     });
                 }
+            } else if (attributable->hasAttribute("fixed_size", true)) {
+                // read offset might be modified by fixed_size's padding. keep it as is.
             } else {
                 evaluator->setBitwiseReadOffset(endOffset);
             }
@@ -399,7 +407,7 @@ namespace pl::core::ast {
 
         if (const auto &arguments = attributable->getAttributeArguments("color"); arguments.size() == 1) {
             auto colorString = getAttributeValueAsString(arguments.front(), evaluator);
-            u32 color = strtoul(colorString.c_str(), nullptr, 16);
+            u32 color = wolv::util::from_chars<u32>(colorString, 16).value_or(0);
             pattern->setColor(hlp::changeEndianess(color, std::endian::big) >> 8);
         } else if (auto singleColor = attributable->hasAttribute("single_color", false); singleColor) {
             pattern->setColor(pattern->getColor());
