@@ -4,6 +4,7 @@
 #include <pl/api.hpp>
 
 #include <optional>
+#include <wolv/utils/charconv.hpp>
 
 namespace pl::core {
     using namespace tkn;
@@ -146,7 +147,7 @@ namespace pl::core {
         m_cursor++; // Skip space
         auto location = this->location();
 
-        while (m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\0') {
+        while (m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\r' && m_sourceCode[m_cursor] != '\0') {
 
             auto character = parseCharacter();
             if (!character.has_value()) {
@@ -170,7 +171,7 @@ namespace pl::core {
 
         while (m_sourceCode[m_cursor] != '\"') {
             char c = peek(0);
-            if (c == '\n') {
+            if (c == '\n' || c == '\r') {
                 m_errorLength = 1;
                 error("Unexpected newline in string literal");
                 m_line++;
@@ -243,10 +244,9 @@ namespace pl::core {
     }
 
     std::optional<double> Lexer::parseFloatingPoint(std::string_view literal, const char suffix) {
-        char *end = nullptr;
-        double val = std::strtod(literal.data(), &end);
+        const auto value = wolv::util::from_chars<double>(literal);
 
-        if(end != literal.data() + literal.size()) {
+        if(!value.has_value()) {
             m_errorLength = literal.size();
             error("Invalid float literal: {}", literal);
             return std::nullopt;
@@ -255,11 +255,11 @@ namespace pl::core {
         switch (suffix) {
             case 'f':
             case 'F':
-                return float(val);
+                return float(*value);
             case 'd':
             case 'D':
             default:
-                return val;
+                return *value;
         }
     }
 
@@ -311,7 +311,7 @@ namespace pl::core {
         m_cursor += 2;
 
         std::string result;
-        while(m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\0') {
+        while(m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\r' && m_sourceCode[m_cursor] != '\0') {
             result += m_sourceCode[m_cursor];
             m_cursor++;
         }
@@ -329,7 +329,7 @@ namespace pl::core {
         m_cursor += 3;
 
         std::string result;
-        while(m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\0') {
+        while(m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\r' && m_sourceCode[m_cursor] != '\0') {
             result += m_sourceCode[m_cursor];
             m_cursor++;
         }
@@ -473,15 +473,11 @@ namespace pl::core {
     hlp::CompileResult<std::vector<Token>> Lexer::lex(const api::Source *source) {
         this->m_sourceCode = source->content;
         this->m_source = source;
-        this->m_cursor = 0;
-        this->m_line = 1;
-        this->m_lineBegin = 0;
-        this->m_longestLineLength = 0;
+
+        this->reset();
 
         const size_t end = this->m_sourceCode.size();
 
-        m_tokens.clear();
-
         while (this->m_cursor < end) {
             const char& c = this->m_sourceCode[this->m_cursor];
 
@@ -651,12 +647,21 @@ namespace pl::core {
 
             m_cursor++;
         }
-
+        m_longestLineLength = std::max(m_longestLineLength, m_cursor - m_lineBegin);
         addToken(makeToken(Separator::EndOfProgram, 0));
 
         return { m_tokens, collectErrors() };
     }
 
+    void Lexer::reset() {
+        this->m_cursor = 0;
+        this->m_line = 1;
+        this->m_lineBegin = 0;
+        this->m_longestLineLength = 0;
+        this->m_tokens.clear();
+    }
+
+
     inline char Lexer::peek(const size_t p) const {
         return m_cursor + p < m_sourceCode.size() ? m_sourceCode[m_cursor + p] : '\0';
     }
@@ -679,4 +684,4 @@ namespace pl::core {
         }
         return Location { m_source, m_line, column, m_errorLength };
     }
-}
\ No newline at end of file
+}
