@@ -178,17 +178,34 @@ namespace pl::core {
         }
 
         if (m_defines.contains(name)) {
-            bool isValueSame = m_defines[name] == values;
+            bool isValueSame = m_defines[name].values == values;
             if (!isValueSame) {
-                errorAt(values[0].location, "Previous definition occurs at line '{}'.", m_defines[name][0].location.line);
-                errorAt(m_defines[name][0].location, "Macro '{}' is redefined in line '{}'.", name, values[0].location.line);
-                m_defines[name].clear();
-                m_defines[name] = values;
+                auto& define = m_defines[name];
+                Location defineLocation{};
+
+                if (define.values.empty()) {
+                    defineLocation = define.nameToken.location;
+                } else {
+                    defineLocation = define.values[0].location;
+                }
+
+                Location ourLocation{};
+                if (values.empty()) {
+                    ourLocation = token.location;
+                } else {
+                    ourLocation = values[0].location;
+                }
+
+                errorAt(ourLocation, "Previous definition occurs at line '{}'.", defineLocation.line);
+                errorAt(defineLocation, "Macro '{}' is redefined in line '{}'.", name, defineLocation.line);
+
+                m_defines[name] = { token, values };
+
                 removeKey(token);
                 m_keys.emplace_back(token);
             }
         } else {
-            m_defines[name] = values;
+            m_defines[name] = { token, values };
             m_keys.emplace_back(token);
         }
     }
@@ -303,6 +320,7 @@ namespace pl::core {
         std::ranges::copy(preprocessor.m_pragmas.begin(), preprocessor.m_pragmas.end(), std::inserter(this->m_pragmas, this->m_pragmas.begin()));
         std::ranges::copy(preprocessor.m_keys.begin(), preprocessor.m_keys.end(), std::inserter(this->m_keys, this->m_keys.begin()));
         std::ranges::copy(preprocessor.m_namespaces.begin(), preprocessor.m_namespaces.end(), std::inserter(this->m_namespaces, this->m_namespaces.begin()));
+        std::ranges::copy(preprocessor.m_parsedImports.begin(), preprocessor.m_parsedImports.end(),std::inserter(this->m_parsedImports, this->m_parsedImports.begin()));
 
         if (shouldInclude) {
             auto content = result.unwrap();
@@ -340,7 +358,7 @@ namespace pl::core {
         if (auto *tokenLiteral = std::get_if<Token::Literal>(&m_token->value); m_token->type == Token::Type::String && tokenLiteral != nullptr) {
             path = tokenLiteral->toString(false);
 
-        } else if (auto *identifier = std::get_if<Token::Identifier>(&m_token->value); m_token->type == Token::Type::Identifier) {
+        } else if (auto *identifier = std::get_if<Token::Identifier>(&m_token->value); m_token->type == Token::Type::Identifier && identifier != nullptr) {
             saveImport.push_back(*m_token);
             path = identifier->get();
             m_token++;
@@ -476,7 +494,7 @@ namespace pl::core {
                         Token::Identifier *tokenIdentifier = std::get_if<Token::Identifier>(&m_token->value);
                         if (tokenIdentifier != nullptr)
                             tokenIdentifier->setType(Token::Identifier::IdentifierType::Macro);
-                        for (const auto &newToken: m_defines[keyIdentifier->get()])
+                        for (const auto &newToken: m_defines[keyIdentifier->get()].values)
                             resultValues.push_back(newToken);
                     } else
                         resultValues.push_back(value);
@@ -519,23 +537,32 @@ namespace pl::core {
         }
     }
 
+    void Preprocessor::reset() {
+        this->m_excludedLocations.clear();
+        this->m_onceIncludedFiles.clear();
+        this->m_onceImportedFiles.clear();
+        this->m_keys.clear();
+        this->m_onlyIncludeOnce = false;
+
+        this->m_defines.clear();
+        this->m_pragmas.clear();
+    }
+
+
     void Preprocessor::appendToNamespaces(std::vector<Token> tokens) {
         for (auto token = tokens.begin(); token != tokens.end(); token++ ) {
             u32 idx = 1;
             if (auto *keyword = std::get_if<Token::Keyword>(&token->value); keyword != nullptr && *keyword == Token::Keyword::Namespace) {
-                if (auto *valueType = std::get_if<Token::ValueType>(&token[1].value);
-                                                        valueType != nullptr && *valueType == Token::ValueType::Auto)
+                if (auto *valueType = std::get_if<Token::ValueType>(&token[1].value); valueType != nullptr && *valueType == Token::ValueType::Auto)
                     idx += 1;
                 auto *identifier = std::get_if<Token::Identifier>(&token[idx].value);
                 while (identifier != nullptr) {
-                    if (auto *separator = std::get_if<Token::Separator>(&token[idx].value);
-                                                        separator != nullptr && *separator == Token::Separator::EndOfProgram)
+                    if (auto *separator = std::get_if<Token::Separator>(&token[idx].value); separator != nullptr && *separator == Token::Separator::EndOfProgram)
                         break;
                     if (std::ranges::find(m_namespaces, identifier->get()) == m_namespaces.end())
                         m_namespaces.push_back(identifier->get());
                     idx += 1;
-                    if (auto *operatorToken = std::get_if<Token::Operator>(&token[idx].value);
-                                                        operatorToken == nullptr || *operatorToken != Token::Operator::ScopeResolution)
+                    if (auto *operatorToken = std::get_if<Token::Operator>(&token[idx].value); operatorToken == nullptr || *operatorToken != Token::Operator::ScopeResolution)
                         break;
                     idx += 1;
                     identifier = std::get_if<Token::Identifier>(&token[idx].value);
@@ -553,13 +580,9 @@ namespace pl::core {
         auto lexer = runtime->getInternals().lexer.get();
 
         if (initialRun) {
-            this->m_excludedLocations.clear();
-            this->m_onceIncludedFiles.clear();
-            this->m_onceImportedFiles.clear();
-            this->m_keys.clear();
-            this->m_onlyIncludeOnce = false;
+            this->reset();
+
 
-            this->m_defines.clear();
             for (const auto& [name, value] : m_runtime->getDefines()) {
                 addDefine(name, value);
             }
@@ -568,7 +591,6 @@ namespace pl::core {
                 addDefine("IMPORTED");
             }
 
-            this->m_pragmas.clear();
             for (const auto& [name, handler]: m_runtime->getPragmas()) {
                 addPragmaHandler(name, handler);
             }
@@ -584,13 +606,14 @@ namespace pl::core {
                 this->error(item);
             return { m_output, collectErrors() };
         }
-
+        setLongestLineLength(lexer->getLongestLineLength());
         m_token = m_result.begin();
         m_initialized = true;
         while (!eof())
             process();
 
         appendToNamespaces(m_output);
+        saveTokens(source, m_result);
 
         // Handle pragmas
         for (const auto &[type, datas] : this->m_pragmas) {
@@ -599,8 +622,7 @@ namespace pl::core {
 
                 if (this->m_pragmaHandlers.contains(type)) {
                     if (!this->m_pragmaHandlers[type](*m_runtime, value))
-                        errorAt(Location { m_source, line, 1, value.length() },
-                                "Value '{}' cannot be used with the '{}' pragma directive.", value, type);
+                        errorAt(Location { m_source, line, 1, value.length() }, "Value '{}' cannot be used with the '{}' pragma directive.", value, type);
                 }
             }
         }
@@ -609,12 +631,20 @@ namespace pl::core {
         return { m_output, collectErrors() };
     }
 
+    void Preprocessor::saveTokens(api::Source *source, const std::vector<Token> &tokens) {
+        if (!source->mainSource && !m_parsedImports.contains(source->source)) {
+            m_parsedImports[source->source] = tokens;
+        }
+    }
+
     bool Preprocessor::eof() {
         return m_token == m_result.end();
     }
 
     void Preprocessor::addDefine(const std::string &name, const std::string &value) {
-        m_defines[name] = {Token { Token::Type::String, value, {nullptr, 0, 0, 0 } } } ;
+        auto nameToken = Token { Token::Type::Identifier, name, Location::Empty() };
+        auto valueToken = Token { Token::Type::String, value, Location::Empty() };
+        m_defines[name] = { nameToken, { valueToken }};
     }
 
     void Preprocessor::addPragmaHandler(const std::string &pragmaType, const api::PragmaHandler &handler) {
